//=============================================================================
// UTHUD.
//=============================================================================
class UTHUD expands UnrealHUD;

var struct EventMsgType
{
	var string MessageStr;
	var float MessageTime;
	var color MessageColor;
} EventMsgs[8];
var() color WhiteColor,StatusMsgColorB,StatusMsgColorL;
var struct SpecialMsgType
{
	var string Msg;
	var name MsgType;
	var float MsgTime;
} SpacialMsgs[8];
var SpecialMsgType PickupMsgs[6];
var float OldTD,ScalingTD,Flashy;
var bool bAdding;
var UTGRI MyGRI;
var color NextMessageColor;

struct LocalMessagesType
{
	var class<LocalMessage> MClass;
	var string MsgTxt;
	var color MsgC;
	var Font MsgF;
	var float MsgT,MsgFT;
};
var LocalMessagesType LocalMessages[30];
var bool bHasLocalMessages;
struct MsgPosBoxType
{
	var float XL,YL,X,Y;
};
var() localized string SpecInfoStr,YouAreOutStr,YouAreDeadStr;

// Static data
var Font LoadedFonts[8];
var() string FontNames[8],UPakFontNames[8];

simulated function PostRender( canvas Canvas )
{
	local float YPos,XL,YL;
	local string S;
	
	if( MyGRI==None )
		MyGRI = Class'DeathMatchPlus'.Static.GetMapGRI(Self);
	if( MyGRI==None )
		Return;
	S = GetStatusMessageStr(MyGRI,UTPRI(Pawn(Owner).PlayerReplicationInfo));
	if( S!="" )
	{
		Canvas.bCenter = true;
		Canvas.Font = GetFont(4);
		Canvas.StrLen("TEST", XL, YL);
		YPos = Canvas.ClipY-YL-33;
		Class'UTDMScoreboard'.Static.RenderNeatBox(Canvas,25,YPos-4,Canvas.ClipX-50,YL+8,StatusMsgColorL,StatusMsgColorB);
		Canvas.DrawColor = WhiteColor;
		Canvas.SetPos(0,YPos);
		Canvas.DrawText(S,False);
	}
	Canvas.Font = Font'WhiteFont';
	if( Canvas.ViewPort.Actor.ViewTarget!=None && Canvas.ViewPort.Actor.ViewTarget!=Owner && Canvas.ViewPort.Actor.ViewTarget.bIsPawn )
	{
		Canvas.bCenter = true;
		Canvas.SetPos(0,Canvas.ClipY/4*3);
		Canvas.DrawColor.R = Flashy*255;
		Canvas.DrawColor.G = Canvas.DrawColor.R;
		Canvas.DrawColor.B = Canvas.DrawColor.R;
		Canvas.Style = ERenderStyle.STY_Translucent;
		Canvas.DrawText("Live feed from"@GetNameOf(Canvas.ViewPort.Actor.ViewTarget));
	}
	Super.PostRender(Canvas);
}
simulated function string GetNameOf( Actor Other )
{
	local Pawn P;
	
	P = Pawn(Other);
	if( P!=None )
	{
		if( P.PlayerReplicationInfo!=None )
			Return P.PlayerReplicationInfo.PlayerName;
		if( P.MenuName!="" )
			Return P.NameArticle$P.MenuName;
		P.MenuName = GetItemName(string(P.Class));
		Return P.NameArticle$P.MenuName;
	}
	Return "a"@GetItemName(string(Other.Class));
}
simulated function Message( PlayerReplicationInfo PRI, coerce string Msg, name N )
{
	local int i;
	
	if( N=='Say' || N=='TeamSay' || N=='Event' || N=='Deathmessage' )
	{
		For( i=7; i>0; i-- )
			EventMsgs[i] = EventMsgs[i-1];
		EventMsgs[0].MessageStr = Msg;
		if( N=='TeamSay' || N=='Say' )
		{
			if( PRI==None )
				EventMsgs[0].MessageStr = "???:"@Msg;
			else EventMsgs[0].MessageStr = PRI.PlayerName$":"@Msg;
		}
		EventMsgs[0].MessageTime = 0;
		if( N=='Event' )
			EventMsgs[0].MessageColor = WhiteColor;
		else if( N=='Say' )
			EventMsgs[0].MessageColor = GreenColor;
		else if( N=='TeamSay' )
		{
			EventMsgs[0].MessageColor = WhiteColor;
			EventMsgs[0].MessageColor.B = 0;
		}
		else EventMsgs[0].MessageColor = RedColor;
	}
	else if( N=='Pickup' || N=='LowCriticalEvent' || N=='MatchMsg' || N=='RedCriticalEvent' || N=='FBMsg' )
	{
		For( i=5; i>0; i-- )
			PickupMsgs[i] = PickupMsgs[i-1];
		PickupMsgs[0].Msg = Msg;
		PickupMsgs[0].MsgType = N;
		PickupMsgs[0].MsgTime = 0;
	}
	else if( N=='CriticalEvent' || N=='SpreeMsg' || N=='WaveMsg' || N=='KillByMsg' || N=='KilledMsg' )
	{
		if( N=='KillByMsg' || N=='KilledMsg' )
		{
			For( i=7; i>0; i-- )
			{
				if( SpacialMsgs[i].MsgType==N )
					SpacialMsgs[i].Msg = "";
			}
		}
		For( i=7; i>0; i-- )
			SpacialMsgs[i] = SpacialMsgs[i-1];
		SpacialMsgs[0].Msg = Msg;
		SpacialMsgs[0].MsgType = N;
		SpacialMsgs[0].MsgTime = 0;
	}
}
simulated function Tick( float D )
{
	local int i;
	local float Ti;
	
	Super.Tick(D);
	if( OldTD!=Level.TimeDilation )
	{
		OldTD = Level.TimeDilation;
		ScalingTD = 1/Level.TimeDilation;
	}
	Ti = D*ScalingTD;
	For( i=0; i<8; i++ )
	{
		if( Len(EventMsgs[i].MessageStr)>0 )
		{
			EventMsgs[i].MessageTime+=Ti;
			if( EventMsgs[i].MessageTime>=6 )
				EventMsgs[i].MessageStr = "";
		}
		if( Len(SpacialMsgs[i].Msg)>0 )
		{
			SpacialMsgs[i].MsgTime+=Ti;
			if( SpacialMsgs[i].MsgTime>=6 )
				SpacialMsgs[i].Msg = "";
		}
		if( i<=5 && Len(PickupMsgs[i].Msg)>0 )
		{
			PickupMsgs[i].MsgTime+=Ti;
			if( PickupMsgs[i].MsgTime>=6 )
				PickupMsgs[i].Msg = "";
		}
	}
	if( bAdding )
	{
		Flashy+=Ti;
		if( Flashy>=1 )
		{
			Flashy = 1;
			bAdding = False;
		}
	}
	else
	{
		Flashy-=Ti;
		if( Flashy<=0 )
		{
			Flashy = 0;
			bAdding = True;
		}
	}
}
simulated function bool DisplayMessages( canvas Canvas )
{
	local float PickupColor,FullX;
	local console Console;
	local int i,Num,ENums[4];
	local float XL,YL,YPos;
	local Font F;

	Console = PlayerPawn(Owner).Player.Console;

	Canvas.Font = Font'WhiteFont';

	DrawTypingPrompt(Canvas, Console);

	if ( !Console.Viewport.Actor.bShowMenu || Console.Viewport.Actor.bShowScores )
	{
		F = Class'UTHUD'.Static.GetFont(4);
		Canvas.bCenter = true;
		Canvas.Style = ERenderStyle.STY_Translucent;
		YPos = Canvas.ClipY/6*5;
		For( i=0; i<6; i++ )
			if ( Len(PickupMsgs[i].Msg)>0 )
			{
				Canvas.SetPos(4,YPos);
				PickupColor = 42.0 * (6-PickupMsgs[i].MsgTime);
				Canvas.DrawColor.r = PickupColor;
				Canvas.DrawColor.g = PickupColor;
				Canvas.DrawColor.b = PickupColor;
				Canvas.Font = Font'WhiteFont';
				if( PickupMsgs[i].MsgType=='LowCriticalEvent' )
				{
					Canvas.DrawColor.r = PickupColor/2;
					Canvas.DrawColor.g = Canvas.DrawColor.r;
					YPos+=YL;
				}
				else if( PickupMsgs[i].MsgType=='MatchMsg' )
				{
					Canvas.DrawColor.r = 0;
					Canvas.DrawColor.g = 0;
					Canvas.Font = F;
				}
				else if( PickupMsgs[i].MsgType=='RedCriticalEvent' )
				{
					Canvas.DrawColor.b = 0;
					Canvas.DrawColor.g = 0;
				}
				else if( PickupMsgs[i].MsgType=='FBMsg' )
				{
					Canvas.DrawColor.b = PickupColor/3;
					Canvas.DrawColor.g = Canvas.DrawColor.b;
					Canvas.Font = F;
				}
				Canvas.StrLen("TEST", XL, YL );
				YPos+=YL;
				Canvas.DrawText(PickupMsgs[i].Msg, true );
			}
		Canvas.Font = Font'WhiteFont';
		Canvas.Style = 1;
		For( i=0; i<8; i++ )
			if( Len(SpacialMsgs[i].Msg)>0 )
			{
				Canvas.DrawColor.r = 0;
				Canvas.DrawColor.g = 128;
				Canvas.DrawColor.b = 255;
				if ( SpacialMsgs[i].MsgType=='CriticalEvent' )
				{
					Canvas.SetPos(0, Console.FrameY/2 - 32+(ENums[0]*9));
					ENums[0]++;
				}
				else if ( SpacialMsgs[i].MsgType=='LowCriticalEvent' ) 
				{
					Canvas.SetPos(0, Console.FrameY/2 + 32+(ENums[1]*9));
					ENums[1]++;
				}
				else if ( SpacialMsgs[i].MsgType=='SpreeMsg' || SpacialMsgs[i].MsgType=='WaveMsg' || SpacialMsgs[i].MsgType=='KillByMsg'
				 || SpacialMsgs[i].MsgType=='KilledMsg' )
				{
					Canvas.Style = ERenderStyle.STY_Translucent;
					PickupColor = 42.0 * (6-SpacialMsgs[i].MsgTime);
					if( SpacialMsgs[i].MsgType=='WaveMsg' )
					{
						Canvas.DrawColor.g = PickupColor;
						Canvas.DrawColor.r = PickupColor;
						Canvas.DrawColor.b = 0;
					}
					else if( SpacialMsgs[i].MsgType=='KillByMsg' )
					{
						Canvas.DrawColor.g = 0;
						Canvas.DrawColor.r = PickupColor;
						Canvas.DrawColor.b = 0;
					}
					else
					{
						Canvas.DrawColor.g = PickupColor/2;
						Canvas.DrawColor.b = PickupColor;
					}
					Canvas.Font = F;
					Canvas.SetPos(0, Canvas.ClipY/5+(ENums[3]*21));
					ENums[3]++;
				}
				Canvas.DrawText( SpacialMsgs[i].Msg, true );
				Canvas.Font = Font'WhiteFont';
				Canvas.Style = 1;
			}
		Canvas.bCenter = false;
		Num = 0;
		Canvas.SetOrigin(ArmorOffset+4,1);
		FullX = Canvas.ClipX;
		Canvas.SetClip(Canvas.ClipX-(ArmorOffset+4),Canvas.ClipY);
		For( i=7; i>=0; i-- )
			if( Len(EventMsgs[i].MessageStr)>0 )
			{
				Canvas.SetPos(0, Num);
				Canvas.StrLen(EventMsgs[i].MessageStr, XL, YL );
				Canvas.DrawColor = EventMsgs[i].MessageColor;
				Canvas.DrawText(EventMsgs[i].MessageStr, false );
				Num+=YL;
			}
		Canvas.SetOrigin(0,0);
		Canvas.SetClip(FullX,Canvas.ClipY);
		if( bHasLocalMessages )
			RenderLocalMessages(Canvas);
	}
	return true;
}

Static function Font GetFont( byte Scale )
{
	if( Scale>=8 )
		Return Font'WhiteFont';
	if( Default.LoadedFonts[Scale]!=None )
		Return Default.LoadedFonts[Scale];
	Default.LoadedFonts[Scale] = Font(DynamicLoadObject("UWindowFonts."$Default.FontNames[Scale],Class'Font',True));
	if( Default.LoadedFonts[Scale]==None )
	{
		Default.LoadedFonts[Scale] = Font(DynamicLoadObject("UWindowFonts."$Default.UPakFontNames[Scale],Class'Font',True));
		if( Default.LoadedFonts[Scale]==None )
			Default.LoadedFonts[Scale] = Font'WhiteFont';
	}
	Return Default.LoadedFonts[Scale];
}
Static function string IntToStr( int Time )
{
	local int Hours, Minutes, Seconds;
	local string HourString, MinuteString, SecondString;
	
	Seconds = Time;
	Minutes = Seconds / 60;
	Hours   = Minutes / 60;
	Seconds = Seconds - (Minutes * 60);
	Minutes = Minutes - (Hours * 60);

	if (Seconds < 10)
		SecondString = "0"$Seconds;
	else
		SecondString = string(Seconds);

	if (Minutes < 10)
		MinuteString = "0"$Minutes;
	else
		MinuteString = string(Minutes);

	if (Hours < 10)
		HourString = "0"$Hours;
	else
		HourString = string(Hours);
	
	Return HourString$":"$MinuteString$":"$SecondString;
}
simulated function DrawFragCount(Canvas Canvas, int X, int Y)
{
	local PlayerReplicationInfo PRI;
	
	if( Canvas.ViewPort.Actor.ViewTarget!=None && Canvas.ViewPort.Actor.ViewTarget.IsA('Pawn') && Pawn(Canvas.ViewPort.Actor.ViewTarget).PlayerReplicationInfo!=None )
		PRI = Pawn(Canvas.ViewPort.Actor.ViewTarget).PlayerReplicationInfo;
	else PRI = Canvas.ViewPort.Actor.PlayerReplicationInfo;
	Canvas.SetPos(X,Y);
	Canvas.DrawIcon(Texture'IconSkull', 1.0);	
	Canvas.CurX -= 19;
	Canvas.CurY += 23;
	if ( PRI==None )
		return;
	Canvas.Font = Font'TinyWhiteFont';
	if (PRI.Score<100) 
		Canvas.CurX+=6;
	if (PRI.Score<10) 
		Canvas.CurX+=6;	
	if (PRI.Score<0) 
		Canvas.CurX-=6;
	if (PRI.Score<-9)
		Canvas.CurX-=6;
	Canvas.DrawText(int(PRI.Score),False);
				
}

simulated function IncomingLocalMsg( Class<LocalMessage> MsgType, object InvolvedObject, int Switch,
 PlayerReplicationInfo InvolvedPRI1, PlayerReplicationInfo InvolvedPRI2, string InvolvedStr, string DesStr )
{
	local int i;
	
	if( !MsgType.Default.bSpecializedMsg )
	{
		For( i=7; i>0; i-- )
			EventMsgs[i] = EventMsgs[i-1];
		EventMsgs[0].MessageStr = DesStr;
		EventMsgs[0].MessageColor = MsgType.Static.GetMessageColor(InvolvedObject,Switch,InvolvedPRI1,InvolvedPRI2,UTPlayer(Owner),InvolvedStr);
		EventMsgs[0].MessageTime = 6.f-MsgType.Static.GetMessageLifeSpan(InvolvedObject,Switch,InvolvedPRI1,InvolvedPRI2,UTPlayer(Owner),InvolvedStr);
		Return;
	}
	// Clear out same message types if desired
	if( MsgType.Default.bReplaceSameMessageTypes )
	{
		For( i=0; i<ArrayCount(LocalMessages); i++ )
		{
			if( LocalMessages[i].MClass!=None && MsgType.Static.ReplaceMessageType(LocalMessages[i].MClass) )
			{
				LocalMessages[i] = LocalMessages[i+1];
				i--;
			}
		}
	}
	// Push away older message types.
	For( i=(ArrayCount(LocalMessages)-1); i>0; i-- )
		LocalMessages[i] = LocalMessages[i-1];
	// Add the new one
	LocalMessages[0].MClass = MsgType;
	LocalMessages[0].MsgTxt = DesStr;
	LocalMessages[0].MsgC = MsgType.Static.GetMessageColor(InvolvedObject,Switch,InvolvedPRI1,InvolvedPRI2
	 ,UTPlayer(Owner),InvolvedStr);
	LocalMessages[0].MsgF = MsgType.Static.GetMessageFont(InvolvedObject,Switch,InvolvedPRI1,InvolvedPRI2
	 ,UTPlayer(Owner),InvolvedStr);
	LocalMessages[0].MsgFT = MsgType.Static.GetMessageLifeSpan(InvolvedObject,Switch,InvolvedPRI1,InvolvedPRI2
	 ,UTPlayer(Owner),InvolvedStr);
	LocalMessages[0].MsgT = Level.TimeSeconds+LocalMessages[0].MsgFT;
	// Notify that there are messages waiting
	bHasLocalMessages = True;
}

// Render the localized messages
simulated function RenderLocalMessages( Canvas C )
{
	local int i,j;
	local bool bHasMessage;
	local MsgPosBoxType Pos[29],Tmp;
	local float F,XL,YL;
	
	// Clean out dead messages first.
	bHasMessage = False;
	For( i=0; i<ArrayCount(LocalMessages); i++ )
	{
		if( LocalMessages[i].MClass!=None )
		{
			if( LocalMessages[i].MsgT<Level.TimeSeconds )
				LocalMessages[i].MClass = None;
			else bHasMessage = True;
		}
	}
	// No new messages anymore, cancel this
	if( !bHasMessage )
	{
		bHasLocalMessages = False;
		Return;
	}
	// Render each message 1 by 1 now
	For( i=0; i<ArrayCount(LocalMessages); i++ )
	{
		if( LocalMessages[i].MClass==None )
			Continue;
		if( LocalMessages[i].MClass.Default.bFadeColorWithLifeSpan )
		{
			F = (LocalMessages[i].MsgT-Level.TimeSeconds)/LocalMessages[i].MsgFT;
			C.DrawColor = LocalMessages[i].MsgC;
			C.DrawColor.R = float(C.DrawColor.R)*F;
			C.DrawColor.G = float(C.DrawColor.G)*F;
			C.DrawColor.B = float(C.DrawColor.B)*F;
		}
		else C.DrawColor = LocalMessages[i].MsgC;
		C.bCenter = LocalMessages[i].MClass.Default.bMessageCentered;
		C.Style = LocalMessages[i].MClass.Default.MessageStyle;
		C.Font = LocalMessages[i].MsgF;
		C.TextSize(LocalMessages[i].MsgTxt,XL,YL);
		Switch( LocalMessages[i].MClass.Default.MessagePivot )
		{
			Case PIV_Center:
				C.CurY = C.ClipY*LocalMessages[i].MClass.Default.MessageCoords.YPos-YL/2;
				C.CurX = C.ClipX*LocalMessages[i].MClass.Default.MessageCoords.XPos-XL/2;
				Break;
			Case PIV_UpLeft:
				C.CurY = C.ClipY*LocalMessages[i].MClass.Default.MessageCoords.YPos-YL;
				C.CurX = C.ClipX*LocalMessages[i].MClass.Default.MessageCoords.XPos-XL;
				Break;
			Case PIV_UpRight:
				C.CurY = C.ClipY*LocalMessages[i].MClass.Default.MessageCoords.YPos-YL;
				C.CurX = C.ClipX*LocalMessages[i].MClass.Default.MessageCoords.XPos;
				Break;
			Case PIV_Up:
				C.CurY = C.ClipY*LocalMessages[i].MClass.Default.MessageCoords.YPos-YL;
				C.CurX = C.ClipX*LocalMessages[i].MClass.Default.MessageCoords.XPos-XL/2;
				Break;
			Case PIV_Down:
				C.CurY = C.ClipY*LocalMessages[i].MClass.Default.MessageCoords.YPos;
				C.CurX = C.ClipX*LocalMessages[i].MClass.Default.MessageCoords.XPos-XL/2;
				Break;
			Case PIV_DownLeft:
				C.CurY = C.ClipY*LocalMessages[i].MClass.Default.MessageCoords.YPos;
				C.CurX = C.ClipX*LocalMessages[i].MClass.Default.MessageCoords.XPos-XL;
				Break;
			Default:
				C.CurY = C.ClipY*LocalMessages[i].MClass.Default.MessageCoords.YPos;
				C.CurX = C.ClipX*LocalMessages[i].MClass.Default.MessageCoords.XPos;
				Break;
		}
		if( C.bCenter )
			C.CurX = 0;
		Tmp = GetBoxCoords(C,LocalMessages[i].MsgTxt,XL,YL);
		if( i>0 )
		{
			For( j=0; j<i; j++ )
			{
				if( Tmp WithIn Pos[j] )
				{
					Tmp.YL+=(Pos[j].Y+0.1f);
					C.CurY = Tmp.YL;
					j = 0;
				}
			}
		}
		if( i<29 )
			Pos[i] = Tmp;
		C.DrawText(LocalMessages[i].MsgTxt,False);
		C.bCenter = False;
	}
}
simulated function MsgPosBoxType GetBoxCoords( Canvas C, string Msg, float XL, float YL )
{
	local MsgPosBoxType B;
	
	B.XL = C.CurX;
	B.YL = C.CurY;
	C.StrLen(Msg,XL,YL);
	if( C.bCenter )
		B.XL = C.ClipX/2-XL/2;
	B.X = XL;
	B.Y = YL;
	Return B;
}
static final operator(24) bool WithIn ( MsgPosBoxType A, MsgPosBoxType B )
{
	Return ((((A.XL+A.X)>=B.XL && A.XL<=B.XL) || ((B.XL+B.X)>=A.XL && B.XL<=A.XL)) &&
	 (((A.YL+A.Y)>=B.YL && A.YL<=B.YL) || ((B.YL+B.Y)>=A.YL && B.YL<=A.YL)));
}
simulated function DrawIdentifyInfo(canvas Canvas, float PosX, float PosY)
{
	local Pawn P;
	local float XL,YL;
	local vector worldPosition,ViewPos,ScrnPos;
	local string TheName;
	local byte Te,MyTeam;
	local UTPlayer UP;

	UP = UTPlayer(Owner);
	if( UP!=None && UP.PlayerReplicationInfo!=None )
		MyTeam = UP.PlayerReplicationInfo.Team;
	else Return;
	ViewPos = Canvas.GetCameraCoords().Origin;
	Canvas.Font = GetFont(0);
	ForEach VisibleCollidingActors(class'Pawn',P,,ViewPos)
	{
		if( P==Owner )
			Continue;
		worldPosition = P.Location;
		worldPosition.Z += P.CollisionHeight+5;
		ScrnPos = Canvas.WorldToScreen(worldPosition,XL);
		if ( ScrnPos.Z>0 )
		{
			if( P.PlayerReplicationInfo!=None )
			{
				Te = P.PlayerReplicationInfo.Team;
				TheName = P.PlayerReplicationInfo.PlayerName;
				if( !UP.PlayerReplicationInfo.bIsSpectator && (Te==255 || Te!=MyTeam) )
					Continue;
			}
			else
			{
				Te = 0;
				if( P.MenuName=="" )
					P.MenuName = string(P.Class.Name);
				TheName = P.MenuName;
			}
			if( Te>3 )
				Te = 0;
			Canvas.DrawColor = Class'UnrealTeamScoreBoard'.Default.TeamColor[Te];
			Canvas.TextSize(TheName,XL,YL);
			Canvas.SetPos(ScrnPos.X - XL/2, ScrnPos.Y - YL);
			Canvas.DrawTextClipped(TheName, false);
			Canvas.TextSize(P.Health,XL,YL);
			Canvas.SetPos(ScrnPos.X - XL/2, ScrnPos.Y);
			Canvas.DrawTextClipped(P.Health, false);
		}
	}
}
simulated function string GetStatusMessageStr( UTGRI MapGRI, UTPRI PLPRI )
{
	if( MapGRI.GameEndedReason!="" )
		Return MapGRI.GameEndedReason;
	else if( PLPRI.bIsSpectator )
		Return SpecInfoStr;
	else if( PLPRI.bIsOutNow )
		Return YouAreOutStr;
	else if( Owner.IsInState('Dying') )
		Return YouAreDeadStr;
	Return "";
}
simulated function DrawInventory(Canvas Canvas, int X, int Y, bool bDrawOne)
{	
	local bool bGotNext, bGotPrev, bGotSelected;
	local inventory Inv,Prev, Next, SelectedItem;
	local translator Translator;
	local int TempX,TempY, HalfHUDX, HalfHUDY, AmmoIconSize, i;
	local Actor InvRendActor;

	if ( HudMode < 4 ) //then draw HalfHUD
	{
		Canvas.Font = Font'TinyFont';
		HalfHUDX = Canvas.ClipX-64;
		HalfHUDY = Canvas.ClipY-32;
		Canvas.CurX = HalfHudX;
		Canvas.CurY = HalfHudY;
		Canvas.DrawIcon(Texture'HalfHud', 1.0);	
	}

	InvRendActor = PlayerPawn(Owner).ViewTarget;
	if( InvRendActor==None )
		InvRendActor = Owner;
	if ( InvRendActor.Inventory==None) Return;
	bGotSelected = False;
	bGotNext = false;
	bGotPrev = false;
	Prev = None;
	Next = None;
	if( InvRendActor.bIsPawn )
		SelectedItem = Pawn(InvRendActor).SelectedItem;

	for ( Inv=InvRendActor.Inventory; Inv!=None; Inv=Inv.Inventory )
	{
		if ( !bDrawOne ) // if drawing more than one inventory, find next and previous items
		{
			if ( Inv == SelectedItem )
				bGotSelected = True;
			else if ( Inv.bActivatable )
			{
				if ( bGotSelected )
				{
					if ( !bGotNext )
					{
						Next = Inv;
						bGotNext = true;
					}
					else if ( !bGotPrev )
						Prev = Inv;
				}
				else
				{
					if ( Next == None )
						Next = Prev;
					Prev = Inv;
					bGotPrev = True;
				}
			}
		}
		
		if ( Translator(Inv) != None )
			Translator = Translator(Inv);

		if ( (HudMode < 4) && (Inv.InventoryGroup>0) && (Weapon(Inv)!=None) ) 
		{
			if( InvRendActor.bIsPawn && Pawn(InvRendActor).Weapon==Inv ) Canvas.Font = Font'TinyWhiteFont';
			else Canvas.Font = Font'TinyFont';
			Canvas.CurX = HalfHudX-3+Inv.InventoryGroup*6;
			Canvas.CurY = HalfHudY+4;
			if (Inv.InventoryGroup<10) Canvas.DrawText(Inv.InventoryGroup,False);
			else Canvas.DrawText("0",False);
		}
		
		
		if ( (HudMode < 4) && (Ammo(Inv)!=None) ) 
		{
			for (i=0; i<10; i++)
			{
				if (Ammo(Inv).UsedInWeaponSlot[i]==1)
				{
					Canvas.CurX = HalfHudX+3+i*6;
					if (i==0) Canvas.CurX += 60;
					Canvas.CurY = HalfHudY+11;
					AmmoIconSize = 16.0*FMin(1.0,(float(Ammo(Inv).AmmoAmount)/float(Ammo(Inv).MaxAmmo)));
					if (AmmoIconSize<8 && Ammo(Inv).AmmoAmount<10 && Ammo(Inv).AmmoAmount>0) 
					{
						Canvas.CurX -= 6;			
						Canvas.CurY += 5;
						Canvas.Font = Font'TinyRedFont';
						Canvas.DrawText(Ammo(Inv).AmmoAmount,False);				
						Canvas.CurY -= 12;
					}
					Canvas.CurY += 19-AmmoIconSize;
					Canvas.CurX -= 6;
					Canvas.DrawColor.g = 255;
					Canvas.DrawColor.r = 0;		
					Canvas.DrawColor.b = 0;					
					if (AmmoIconSize<8) 
					{
						Canvas.DrawColor.r = 255-AmmoIconSize*30;
						Canvas.DrawColor.g = AmmoIconSize*30+40;				
					}
					if (Ammo(Inv).AmmoAmount >0) 
					{
						Canvas.DrawTile(Texture'HudGreenAmmo',4.0,AmmoIconSize,0,0,4.0,AmmoIconSize);		
					}
					Canvas.DrawColor.g = 255;
					Canvas.DrawColor.r = 255;		
					Canvas.DrawColor.b = 255;	
				}
			}
		}
	}

	// List Translator messages if activated
	if ( Translator!=None )
	{
		if( Translator.bCurrentlyActivated )
		{
			Canvas.bCenter = false;
			Canvas.Font = Canvas.MedFont;
			TempX = Canvas.ClipX;
			TempY = Canvas.ClipY;
			CurrentMessage = Translator.NewMessage;
			Canvas.Style = 2;	
			Canvas.SetPos(Canvas.ClipX/2-128, Canvas.ClipY/2-68);
			Canvas.DrawIcon(texture'TranslatorHUD3', 1.0);
			Canvas.SetOrigin(Canvas.ClipX/2-110,Canvas.ClipY/2-52);
			Canvas.SetClip(225,110);
			Canvas.SetPos(0,0);
			Canvas.Style = 1;	
			Canvas.DrawText(CurrentMessage, False);	
			HUDSetup(canvas);	
			Canvas.ClipX = TempX;
			Canvas.ClipY = TempY;
		}
		else 
			bFlashTranslator = ( Translator.bNewMessage || Translator.bNotNewMessage );
	}

	if ( HUDMode == 5 )
		return;

	if ( SelectedItem != None )
	{	
		Count++;
		if (Count>20) Count=0;
		
		if (Prev!=None) 
		{
			if ( Prev.bActive || (bFlashTranslator && (Translator == Prev) && (Count>15)) )
			{
				Canvas.DrawColor.b = 0;		
				Canvas.DrawColor.g = 0;		
			}
			DrawHudIcon(Canvas, X, Y, Prev);				
			if ( (Pickup(Prev) != None) && Pickup(Prev).bCanHaveMultipleCopies )
				DrawNumberOf(Canvas,Pickup(Prev).NumCopies,X,Y);
			Canvas.DrawColor.b = 255;
			Canvas.DrawColor.g = 255;		
		}
		if ( SelectedItem.Icon != None )	
		{
			if ( SelectedItem.bActive || (bFlashTranslator && (Translator == SelectedItem) && (Count>15)) )
			{
				Canvas.DrawColor.b = 0;		
				Canvas.DrawColor.g = 0;		
			}
			if ( (Next==None) && (Prev==None) && !bDrawOne) DrawHudIcon(Canvas, X+64, Y, SelectedItem);
			else DrawHudIcon(Canvas, X+32, Y, SelectedItem);		
			Canvas.Style = 2;
			Canvas.CurX = X+32;
			if ( (Next==None) && (Prev==None) && !bDrawOne ) Canvas.CurX = X+64;
			Canvas.CurY = Y;
			Canvas.DrawIcon(texture'IconSelection', 1.0);
			if ( (Pickup(SelectedItem) != None) 
				&& Pickup(SelectedItem).bCanHaveMultipleCopies )
				DrawNumberOf(Canvas,Pickup(SelectedItem).NumCopies,Canvas.CurX-32,Y);
			Canvas.Style = 1;
			Canvas.DrawColor.b = 255;
			Canvas.DrawColor.g = 255;		
		}
		if (Next!=None) {
			if ( Next.bActive || (bFlashTranslator && (Translator == Next) && (Count>15)) )
			{
				Canvas.DrawColor.b = 0;		
				Canvas.DrawColor.g = 0;		
			}
			DrawHudIcon(Canvas, X+64, Y, Next);
			if ( (Pickup(Next) != None) && Pickup(Next).bCanHaveMultipleCopies )
				DrawNumberOf(Canvas,Pickup(Next).NumCopies,Canvas.CurX-32,Y);
			Canvas.DrawColor.b = 255;
			Canvas.DrawColor.g = 255;
		}
	}
}
simulated function DrawArmor(Canvas Canvas, int X, int Y, bool bDrawOne)
{
	Local int ArmorAmount,CurAbs;
	Local inventory Inv,BestArmor;
	Local float XL, YL;
	local Actor RendAct;
	local Adrenaline Adr;

	RendAct = PlayerPawn(Owner).ViewTarget;
	if( RendAct==None )
		RendAct = Owner;
	ArmorAmount = 0;
	ArmorOffset = 0;
	Canvas.Font = Canvas.LargeFont;
	Canvas.CurX = X;
	Canvas.CurY = Y;
	CurAbs=0;
	BestArmor=None;
	for( Inv=RendAct.Inventory; Inv!=None; Inv=Inv.Inventory ) 
	{
		if( Adrenaline(Inv)!=None )
		{
			Adr = Adrenaline(Inv);
			Continue;
		}
		if (Inv.bIsAnArmor) 
		{
			ArmorAmount += Inv.Charge;				
			if (Inv.Charge>0 && Inv.Icon!=None) 
			{
				if (!bDrawOne) 
				{
					ArmorOffset += 32;
					DrawHudIcon(Canvas, Canvas.CurX, Y, Inv);
					DrawIconValue(Canvas, Inv.Charge);						
				}
				else if (Inv.ArmorAbsorption>CurAbs) 
				{
					CurAbs = Inv.ArmorAbsorption;
					BestArmor = Inv;
				}
			}
		}
	}
	if( Adr!=None )
	{
		if( ArmorOffset<37 )
			ArmorOffset = 37;
		Canvas.SetPos(5,34);
		DrawHudIcon(Canvas, Canvas.CurX, Canvas.CurY, Adr);
		Canvas.SetPos(4,66);
		Canvas.DrawText(Adr.Charge,False);
	}
	if (bDrawOne && BestArmor!=None) 
	{
		DrawHudIcon(Canvas, Canvas.CurX, Y, BestArmor);
		DrawIconValue(Canvas, BestArmor.Charge);		
	}
	Canvas.CurY = Y;
	if (ArmorAmount>0 && HudMode==0) {
		Canvas.StrLen(ArmorAmount,XL,YL);
		ArmorOffset += XL;
		Canvas.DrawText(ArmorAmount,False);	
	}
}
simulated function DrawAmmo(Canvas Canvas, int X, int Y)
{
	local Actor RendAct;

	RendAct = PlayerPawn(Owner).ViewTarget;
	if( RendAct==None )
		RendAct = Owner;
	if ( !RendAct.bIsPawn || (Pawn(RendAct).Weapon == None) || (Pawn(RendAct).Weapon.AmmoType == None) )
		return;
	Canvas.CurY = Y;
	Canvas.CurX = X;
	Canvas.Font = Canvas.LargeFont;
	if( Pawn(RendAct).Weapon.AmmoType.AmmoAmount<10 ) Canvas.Font = Font'LargeRedFont';	
	if (HudMode==0) {
		if (Pawn(RendAct).Weapon.AmmoType.AmmoAmount>=100) Canvas.CurX -= 16;
		if (Pawn(RendAct).Weapon.AmmoType.AmmoAmount>=10) Canvas.CurX -= 16;
		Canvas.DrawText(Pawn(RendAct).Weapon.AmmoType.AmmoAmount,False);
		Canvas.CurY = Canvas.ClipY-32;
	}
	else Canvas.CurX+=16;
	if (Pawn(RendAct).Weapon.AmmoType.Icon!=None) Canvas.DrawIcon(Pawn(RendAct).Weapon.AmmoType.Icon, 1.0);
	Canvas.CurY += 29;
	DrawIconValue(Canvas, Pawn(RendAct).Weapon.AmmoType.AmmoAmount);
	Canvas.CurX = X+19;
	Canvas.CurY = Y+29;
	if (HudMode!=1 && HudMode!=2 && HudMode!=4)  Canvas.DrawTile(Texture'HudLine',
		FMin(27.0*(float(Pawn(RendAct).Weapon.AmmoType.AmmoAmount)/float(Pawn(RendAct).Weapon.AmmoType.MaxAmmo)),27),2.0,0,0,32.0,2.0);
}
simulated function DrawHealth(Canvas Canvas, int X, int Y)
{
	local Actor RendAct;

	RendAct = PlayerPawn(Owner).ViewTarget;
	if( RendAct==None )
		RendAct = Owner;
	if( !RendAct.bIsPawn )
		Return;
	Canvas.CurY = Y;
	Canvas.CurX = X;	
	Canvas.Font = Canvas.LargeFont;
	if (Pawn(RendAct).Health<25) Canvas.Font = Font'LargeRedFont';
	Canvas.DrawIcon(Texture'IconHealth', 1.0);
	Canvas.CurY += 29;	
	DrawIconValue(Canvas, Max(0,Pawn(RendAct).Health));
	Canvas.CurY -= 29;		
	if (HudMode==0) Canvas.DrawText(Max(0,Pawn(RendAct).Health),False);	
	Canvas.CurY = Y+29;		
	Canvas.CurX = X+2;
	if (HudMode!=1 && HudMode!=2 && HudMode!=4) 
		Canvas.DrawTile(Texture'HudLine',FMin(27.0*(float(Pawn(RendAct).Health)/float(Pawn(RendAct).Default.Health)),27),2.0,0,0,32.0,2.0);	
}
simulated function GetMessageColor( name MsgType, out color Colour )
{
	switch( MsgType )
	{
	case 'Local':
		Colour = NextMessageColor;
		NextMessageColor = Default.NextMessageColor;
		break;
	case 'WaveMsg':
		Colour.R = 255;
		Colour.G = 255;
		Colour.B = 0;
		break;
	case 'MatchMsg':
		Colour.R = 0;
		Colour.G = 0;
		Colour.B = 200;
		break;
	case 'SpreeMsg':
	case 'KilledMsg':
		Colour.R = 0;
		Colour.G = 125;
		Colour.B = 250;
		break;
	case 'KillByMsg':
		Colour.R = 230;
		Colour.G = 0;
		Colour.B = 0;
		break;
	case 'FBMsg':
		Colour.R = 150;
		Colour.G = 0;
		Colour.B = 0;
		break;
	default:
		Super.GetMessageColor(MsgType,Colour);
	}
}

defaultproperties
{
				WhiteColor=(R=255,G=255,B=255)
				StatusMsgColorB=(R=60,G=77,B=108)
				StatusMsgColorL=(R=90,G=81,B=145)
				NextMessageColor=(R=255,G=255,B=255)
				SpecInfoStr="You are spectating, press [Fire] to view next player."
				YouAreOutStr="You are out, press [Fire] to spectate next player."
				YouAreDeadStr="You are dead, press [Fire] to respawn."
				FontNames(0)="Tahoma10"
				FontNames(1)="UTFont12"
				FontNames(2)="Tahoma15"
				FontNames(3)="Tahoma17"
				FontNames(4)="UTFont24"
				FontNames(5)="Tahoma22"
				FontNames(6)="Tahoma24"
				FontNames(7)="UTFont40"
				UPakFontNames(0)="Tahoma10"
				UPakFontNames(1)="UTFont12"
				UPakFontNames(2)="Tahoma10"
				UPakFontNames(3)="Tahoma20"
				UPakFontNames(4)="UTFont24"
				UPakFontNames(5)="Tahoma20"
				UPakFontNames(6)="Tahoma20"
				UPakFontNames(7)="UTFont40"
				bAlwaysTick=True
}
